/*
	This file is the main C function for a C program that analyzes stocks
	and then chooses the best 3 stocks based on the analysis performed.
	This file contains the main method of the program that is responsible
	for input/file handling, as well as all the function calls that are
	used in order to determine the top 3 stock choices.
	
	Alex Dobson
	CSC 370, University Of Alberta: Augustana Campus
	Fall 2012 
*/

#include <stdio.h>
#include "equity.h"

#define NUM_STOCKS 100
#define TOP_STOCKS 3

typedef struct {
	char name[9];
	double price;
	double esp;
	double dividend;
	long yesterdaySales;
	long todaySales;
	double yield;
	double peRatio;
	double volumeTrend;
	int recommend;
} stock;

void readData();
void sortStocks();
void output();
inline void swap( stock *, stock * );

// Globals, because its the C way
FILE *file ;
stock stocks[NUM_STOCKS];
int stock_count;


/*
	"main()" will always run first. Function is used to handle the input from the command 
	line which can be formatted in one of two ways. First, if the program is run with no 
	parameters, then input will come from the standard input (ie the command line). The 
	file pointer in this case is set to stdin, and readData is called on the stdin. If 
	command line arguments are ran with the program, the those are assumed to be files that 
	contain stock data. For each command line argument passed in in this case, the file 
	pointer is set to that particular file (at which time it is checked to make sure it is 
	not null) and then readData is called on each individual file.
*/
int main( int argc, char *argv[] ) {
	int i; //for loop int
	
	// if no arguments were passed in, ie ./analysis
	if ( argc == 1 ){
		printf( "No parameters given, please enter the stocks in the command line and press Ctrl-D when finished. \n");
		file = stdin;
		readData();
	}
	
	// if arguments were passed in, ie ./analysis stocks_1.txt stocks_2.txt ...
	else {
		for ( i = 1; i < argc; i++ ) {
			if ( ( file = fopen( argv[i], "r" ) ) != NULL ) {
				readData();
			} else {
				fprintf( stderr, "FILE '%s' COULD NOT BE READ \n", argv[i] );
			}
		}
	}
	
	sortStocks();
	output();
	
	return 0;
	}// end of Main
	

/*
	Function is used to read in data line by line from the file pointer passed by main
	and to parse the data in each line into stock information. This data about each 
	stock (which is assumed to be written in the correct format on each line) is then
	stored in the stocks[] array. With each element added, the stock_count is incremented.
	
	GLOBALS MODIFIED: stocks[], stock_count
*/
void readData() {
	while ( fscanf( file, "%8s %lf %lf %lf %ld %ld", stocks[stock_count].name, &stocks[stock_count].price, 
			&stocks[stock_count].esp, &stocks[stock_count].dividend, &stocks[stock_count].yesterdaySales, 
			&stocks[stock_count].todaySales ) != EOF && stock_count < 101 ) {
			
		stocks[stock_count].yield = yield( stocks[stock_count].price, stocks[stock_count].dividend );
		stocks[stock_count].peRatio = peRatio( stocks[stock_count].price, stocks[stock_count].esp );
		stocks[stock_count].volumeTrend = volumeTrend( stocks[stock_count].yesterdaySales, stocks[stock_count].todaySales );
		
		//printf( "%s %.2lf %.2lf %.2lf %ld %ld %.2lf %.2lf %.2lf \n", stocks[stock_count].name, stocks[stock_count].price,
		//	stocks[stock_count].esp, stocks[stock_count].dividend, stocks[stock_count].yesterdaySales, stocks[stock_count].todaySales, 
		//	stocks[stock_count].yield, stocks[stock_count].peRatio, stocks[stock_count].volumeTrend );
		
		if ( stocks[stock_count].yield > 3 && stocks[stock_count].peRatio < 12 && stocks[stock_count].volumeTrend > -1 ) {
			stocks[stock_count].recommend = 0; // Recommend Stock
		} else {
			stocks[stock_count].recommend = 1; // Do Not Recommend Stock
		}
		
		stock_count++;
		
	}
			
	fclose( file );
	
}//end of readData()


/*
	Sorts the stocks in the stocks[] array using a bubble sort according to the
	stocks yield. The stocks are swapped using swap so that the one with the biggest
	yield is in the first location in the array. This function is taken from the local
	folder on the Augustana Savage server.
	
	GLOBALS MODIFIED: stocks[]
	
	REF: usr/local/courses/aucsc370/C/bubblesort.c on the Augustana Savage Server.
*/
void sortStocks() {
	int i, length, sorted;
	
	length = stock_count;
	sorted = 1; // false
	do {
		sorted = 0; // true
		length--;
		for ( i = 0; i < length; i++ ) {
			if ( stocks[i].yield < stocks[i+1].yield ){
				sorted = 1; // false
				swap( stocks+i, stocks+i+1 );
			}
		}
	} while ( sorted != 0 ); // while( !sorted )
}//end of void sortStocks()


/*
	Swaps two stocks in the stocks[] array using a temporary stock. This function is
	taken from the local folder on the Augustana Savage server.
	
	REF: /usr/local/courses/aucsc370/C/bubblesort.c
*/
inline void swap( stock *s1, stock *s2 ) {
	stock temp = *s1;
	*s1 = *s2;
	*s2 = temp;
}//end of inline void swap()


/*
	Prints off the TOP_STOCKS (TOP_STOCKS is a number) from the stocks[] array. The
	top stocks printed must be recommended stocks in order to be printed. Once the 
	top stocks are printed the function returns.
*/
void output() {
	int i, num_found = 0;
	
	printf( "\n+-------------------------------- TOP %d STOCKS --------------------------------+ \n", TOP_STOCKS );
	printf( "| Rank  | Name      |          Yield |   Price/Earnings Ratio |   Volume Trend | \n" );
	printf( "+------------------------------------------------------------------------------+ \n" );
	for ( i = 0; i < stock_count; i++ ) {
		if ( stocks[i].recommend == 0 ) {
			printf( "|  # %d  | %-8s  |  %10.2lf %%  |  %20.2lf  |  %10.2lf %%  | \n", 
				num_found + 1, stocks[i].name, stocks[i].yield, stocks[i].peRatio, stocks[i].volumeTrend );
			num_found++;
		}
		// if TOP_STOCKS (TOP_STOCKS is a number) stocks are already found then return
		if ( num_found >= TOP_STOCKS ) {
			printf( "+------------------------------------------------------------------------------+ \n" ); 
			return;
		}
	}
	printf( "| \t \t \t    * No Recommended Stocks *  \t \t \t       | \n" );
	printf( "+------------------------------------------------------------------------------+ \n" );
}//end of void topStocks()